-- Package declaration
local _runner = _G["github.com/eandre/groupauras/core/runner"] or {}
_G["github.com/eandre/groupauras/core/runner"] = _runner

local builtins = _G.lunar_go_builtins

local _aura = _G["github.com/eandre/groupauras/core/aura"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_runner.Runner = {}

_runner.Runner._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_runner.Runner})
	local obj, err
	
	self.running = nil
	if type(self.running) == type(tbl.running) then self.running = tbl.running end
	self.updateMap = nil
	if type(self.updateMap) == type(tbl.updateMap) then self.updateMap = tbl.updateMap end
	self.eventMap = nil
	if type(self.eventMap) == type(tbl.eventMap) then self.eventMap = tbl.eventMap end
	return self, nil
end

_runner.Runner._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.running) == type(tbl.running) then self.running = tbl.running end
	if type(self.updateMap) == type(tbl.updateMap) then self.updateMap = tbl.updateMap end
	if type(self.eventMap) == type(tbl.eventMap) then self.eventMap = tbl.eventMap end
	return nil
end

_runner.New = function()
	return setmetatable({ ["running"] = {}, ["updateMap"] = {}, ["eventMap"] = {} }, {__index=_runner.Runner})
end

_runner.Runner.Running = function(r, ca)
	return (r.running[ca] or false)
end

_runner.Runner.Start = function(r, ca)
	if (r.running[ca] or false) then
		return
	end
	r.running[ca] = true
	if ca.OnActivate ~= nil then
		ca.OnActivate(ca)
	end
	if ca.OnUpdate ~= nil then
		r.updateMap[ca] = true
		_wow.RegisterUpdate(builtins.create_closure(r, "onUpdate"))
	end
	for event in pairs(ca.Events or {}) do
		local m = (r.eventMap[event] or nil)
		if m == nil then
			m = {}
			_wow.RegisterEvent(event, builtins.create_closure(r, "onEvent"))
		end
		m[ca] = true
	end
end

_runner.Runner.Stop = function(r, ca)
	if (not (r.running[ca] or false)) then
		return
	end
	builtins.delete(r.running, ca)
	for event in pairs(ca.Events or {}) do
		local m = (r.eventMap[event] or nil)
		builtins.delete(m, ca)
		if builtins.mapLength(m) == 0 then
			_wow.UnregisterEvent(event, builtins.create_closure(r, "onEvent"))
			builtins.delete(r.eventMap, event)
		end
	end
	if ca.OnUpdate ~= nil then
		builtins.delete(r.updateMap, ca)
		if builtins.mapLength(r.updateMap) == 0 then
			_wow.UnregisterUpdate(builtins.create_closure(r, "onUpdate"))
		end
	end
	if ca.OnDeactivate ~= nil then
		ca.OnDeactivate(ca)
	end
end

_runner.Runner.onEvent = function(r, event, args)
	for ca in pairs((r.eventMap[event] or nil) or {}) do
		(ca.Events[event] or nil)(ca, event, args)
	end
end

_runner.Runner.onUpdate = function(r, dt)
	for ca in pairs(r.updateMap or {}) do
		ca.OnUpdate(ca, dt)
	end
end

