-- Package declaration
local _draw = _G["github.com/eandre/groupauras/pkg/draw"] or {}
_G["github.com/eandre/groupauras/pkg/draw"] = _draw

local builtins = _G.lunar_go_builtins

local _raidutil = _G["github.com/eandre/groupauras/pkg/raidutil"]
local _hbd = _G["github.com/eandre/lunar-shim/hbd"]
local _luastrings = _G["github.com/eandre/lunar-wow/pkg/luastrings"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]


_draw.PlayerPosition = function()
	return setmetatable({  }, {__index=_draw.playerPosition})
end

_draw.UnitPosition = function(unitID)
	if _luastrings.HasPrefix((unitID), "raid") or _luastrings.HasPrefix((unitID), "party") then
		local guid = _wow.UnitGUID(unitID)
		return setmetatable({ ["guid"] = guid }, {__index=_draw.guidPosition})
	end
	return setmetatable({ ["unitID"] = unitID }, {__index=_draw.unitIDPosition})
end

_draw.GUIDPosition = function(guid)
	return setmetatable({ ["guid"] = guid }, {__index=_draw.guidPosition})
end

_draw.StaticPlayerPosition = function()
	local x, y, inst = _hbd.PlayerWorldPosition()
	return setmetatable({ ["x"] = x, ["y"] = y, ["inst"] = inst }, {__index=_draw.staticPosition})
end

_draw.StaticUnitPosition = function(unitID)
	local x, y, inst = _hbd.UnitWorldPosition(unitID)
	return setmetatable({ ["x"] = x, ["y"] = y, ["inst"] = inst }, {__index=_draw.staticPosition})
end

_draw.staticPosition = {}

_draw.staticPosition._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.staticPosition})
	local obj, err
	
	self.x = 0
	if type(self.x) == type(tbl.x) then self.x = tbl.x end
	self.y = 0
	if type(self.y) == type(tbl.y) then self.y = tbl.y end
	self.inst = 0
	if type(self.inst) == type(tbl.inst) then self.inst = tbl.inst end
	return self, nil
end

_draw.staticPosition._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.x) == type(tbl.x) then self.x = tbl.x end
	if type(self.y) == type(tbl.y) then self.y = tbl.y end
	if type(self.inst) == type(tbl.inst) then self.inst = tbl.inst end
	return nil
end

_draw.staticPosition.Pos = function(sp)
	return sp.x, sp.y, sp.inst
end

_draw.staticPosition.Static = function(sp)
	return true
end

_draw.playerPosition = {}

_draw.playerPosition._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.playerPosition})
	local obj, err
	
	return self, nil
end

_draw.playerPosition._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	return nil
end

_draw.playerPosition.Static = function(pp)
	return false
end

_draw.playerPosition.Pos = function(pp)
	return _hbd.PlayerWorldPosition()
end

_draw.unitIDPosition = {}

_draw.unitIDPosition._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.unitIDPosition})
	local obj, err
	
	self.unitID = ""
	if type(self.unitID) == type(tbl.unitID) then self.unitID = tbl.unitID end
	return self, nil
end

_draw.unitIDPosition._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.unitID) == type(tbl.unitID) then self.unitID = tbl.unitID end
	return nil
end

_draw.unitIDPosition.Static = function(up)
	return false
end

_draw.unitIDPosition.Pos = function(up)
	return _hbd.UnitWorldPosition(up.unitID)
end

_draw.guidPosition = {}

_draw.guidPosition._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_draw.guidPosition})
	local obj, err
	
	self.guid = ""
	if type(self.guid) == type(tbl.guid) then self.guid = tbl.guid end
	return self, nil
end

_draw.guidPosition._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.guid) == type(tbl.guid) then self.guid = tbl.guid end
	return nil
end

_draw.guidPosition.Static = function(up)
	return false
end

_draw.guidPosition.Pos = function(up)
	local uid, ok = _raidutil.GUIDToUnitID(up.guid)
	if (not ok) then
		return 0, 0, 0
	end
	return _hbd.UnitWorldPosition(uid)
end

