-- Package declaration
local _auratrack = _G["github.com/eandre/groupauras/pkg/raidutil/auratrack"] or {}
_G["github.com/eandre/groupauras/pkg/raidutil/auratrack"] = _auratrack

local builtins = _G.lunar_go_builtins

local _context = _G["github.com/eandre/groupauras/pkg/context"]
local _wow = _G["github.com/eandre/lunar-wow/pkg/wow"]

_auratrack.Target = {}

_auratrack.Target._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_auratrack.Target})
	local obj, err
	
	self.GUID = ""
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	self.Name = ""
	if type(self.Name) == type(tbl.Name) then self.Name = tbl.Name end
	self.Self = false
	if type(self.Self) == type(tbl.Self) then self.Self = tbl.Self end
	self.cancelFunc = nil
	if type(self.cancelFunc) == type(tbl.cancelFunc) then self.cancelFunc = tbl.cancelFunc end
	return self, nil
end

_auratrack.Target._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.GUID) == type(tbl.GUID) then self.GUID = tbl.GUID end
	if type(self.Name) == type(tbl.Name) then self.Name = tbl.Name end
	if type(self.Self) == type(tbl.Self) then self.Self = tbl.Self end
	if type(self.cancelFunc) == type(tbl.cancelFunc) then self.cancelFunc = tbl.cancelFunc end
	return nil
end

_auratrack.Tracker = {}

_auratrack.Tracker._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_auratrack.Tracker})
	local obj, err
	
	self.Targets = nil
	if type(self.Targets) == type(tbl.Targets) then self.Targets = tbl.Targets end
	self.spellID = 0
	if type(self.spellID) == type(tbl.spellID) then self.spellID = tbl.spellID end

		obj, err = _auratrack.Cfg._createFromTable(tbl.cfg)
		if err ~= nil then
			return nil, err
		end
		self.cfg = obj
					
	return self, nil
end

_auratrack.Tracker._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.Targets) == type(tbl.Targets) then self.Targets = tbl.Targets end
	if type(self.spellID) == type(tbl.spellID) then self.spellID = tbl.spellID end

		obj, err = _auratrack.Cfg._createFromTable(tbl.cfg)
		if err ~= nil then
			return err
		end
		self.cfg = obj
					
	return nil
end

_auratrack.Cfg = {}

_auratrack.Cfg._createFromTable = function(tbl)
	if tbl == nil then
		return nil, nil
	end
	if type(tbl) ~= "table" then
		return nil, builtins.create_error("cannot initialize struct from non-table")
	end
	local self = setmetatable({}, {__index=_auratrack.Cfg})
	local obj, err
	
	self.Add = nil
	if type(self.Add) == type(tbl.Add) then self.Add = tbl.Add end
	self.Remove = nil
	if type(self.Remove) == type(tbl.Remove) then self.Remove = tbl.Remove end
	return self, nil
end

_auratrack.Cfg._initializeFromTable = function(self, tbl)
	if tbl == nil then
		return nil
	end
	if type(tbl) ~= "table" then
		return builtins.create_error("cannot initialize struct from non-table")
	end
	local obj, err
		
	if type(self.Add) == type(tbl.Add) then self.Add = tbl.Add end
	if type(self.Remove) == type(tbl.Remove) then self.Remove = tbl.Remove end
	return nil
end

_auratrack.New = function(ctx, spellID, cfg)
	local t = setmetatable({ ["Ctx"] = ctx, ["Targets"] = {}, ["spellID"] = spellID, ["cfg"] = cfg }, {__index=_auratrack.Tracker})
	t:enable()
	ctx:OnCancel(t, _auratrack.disableTracker)
	return t
end

_auratrack.ongoingTrackers = {}

_auratrack.disableTracker = function(ctx, data)
	data:disable()
end

_auratrack.Tracker.enable = function(t)
	if (_auratrack.ongoingTrackers[t.spellID] or nil) == nil then
		_auratrack.ongoingTrackers[t.spellID] = {}
	end
	(_auratrack.ongoingTrackers[t.spellID] or nil)[t] = true
end

_auratrack.Tracker.disable = function(t)
	builtins.delete((_auratrack.ongoingTrackers[t.spellID] or nil), t)
end

builtins.add_init(function()
	local selfGUID = _wow.UnitGUID("player")
	_wow.RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED", function(_, args)
		local event = (args[1 + 1] or nil)
		if event ~= "SPELL_AURA_APPLIED" and event ~= "SPELL_AURA_REMOVED" then
			return
		end
		local spellID = (args[11 + 1] or nil)
		local trackers = (_auratrack.ongoingTrackers[spellID] or nil)
		if builtins.mapLength(trackers) == 0 then
			return
		end
		local destGUID = (args[7 + 1] or nil)
		if event == "SPELL_AURA_APPLIED" then
			for t in pairs(trackers or {}) do
				local ctx, cancel = _context.New(t.Ctx)
				local target = { ["Ctx"] = ctx, ["cancelFunc"] = cancel, ["GUID"] = destGUID, ["Name"] = (args[8 + 1] or nil), ["Self"] = destGUID == selfGUID}
				t.Targets[destGUID] = target
				if t.cfg.Add ~= nil then
					t.cfg.Add(target)
				end
			end
			return
		end
		for t in pairs(trackers or {}) do
			local target = (t.Targets[destGUID] or nil)
			if target ~= nil then
				builtins.delete(t.Targets, destGUID)
				target.cancelFunc()
				if t.cfg.Remove ~= nil then
					t.cfg.Remove(target)
				end
			end
		end
	end)
end)

